class Chef
  class Resource
    class SystemdKmod < Chef::Resource
      identity_attr :name

      def initialize(name, run_context = nil)
        super
        @resource_name = :systemd_kmod
        @provider = Chef::Provider::SystemdKmod
        @action = :load
        @allowed_actions = [:load, :unload, :blacklist, :autoload]
        @name = name
        @returns = 0
        @options = nil
        @loaded = false
      end

      def options(arg = nil)
        set_or_return(:options, arg, kind_of: [Hash, Mash])
      end
    end
  end
end

class Chef
  class Provider
    class SystemdKmod < Chef::Provider
      # implement load_current_resource method to load previous resource before action
      def load_current_resource
        @current_resource = Chef::Resource::SystemdKmod.new(@new_resource.name)
        @current_resource.name(@new_resource.name)
        @current_resource.options(@new_resource.options)
        @current_resource
      end

      def path_boot
        case node['platform']
        when 'exherbo', 'ubuntu', 'arch'
          return "/etc/modules-load.d/#{new_resource.name}.conf"
        when 'debian', 'ubuntu'
          return '/etc/modules'
        end
      end

      def unblacklist
        if ::File.exist?(path_opts)
          if ::File.read(path_opts) == "install #{new_resource.name} /bin/true\n"
            ::File.delete(path_opts)
          end
        end
      end

      def need_reload
        current = Mash.new
        target = Mash.new

        unless ::File.exist?("/sys/module/#{new_resource.name}")
          return false
        end

        path = ::File.join('/sys/module', new_resource.name, 'parameters')
        Dir.foreach(path) do |param|
          next if param == '.' or param == '..'

          contents = ::File.read(::File.join(path, param)).chomp!
          case contents
          when 'Y'
            contents = true
          when 'N'
            contents = false
          end

          current[param] = contents
        end

        target = Chef::Mixin::DeepMerge.merge(current, new_resource.options)

        if target == current
          return false
        else
          return true
        end
      end


      def path_opts
        '/etc/modprobe.d/#{new_resource.name}.conf'
      end

      def serialize
        output = ' '
        unless new_resource.options.nil?
          new_resource.options.each do |option, value|
            output << " #{option}=\"#{value}\""
          end
        end
        output
      end

      def action_load
        unblacklist
        if need_reload
          action_unload
        end
        s = file path_opts do
          content "options #{new_resource.name}#{serialize}\n"
          owner 'root'
          group 'root'
          mode '0644'
          not_if { new_resource.options.nil? }
        end
        execute "load module #{new_resource.name}" do
          command "modprobe -b #{new_resource.name}#{serialize}"
          not_if { ::File.exist?("/sys/module/#{new_resource.name}") }
        end
        new_resource.updated_by_last_action(true) if  s.updated_by_last_action?
      end

      def action_autoload
        b = file path_boot do
          content new_resource.name
          owner 'root'
          group 'root'
          mode '0644'
        end
        new_resource.updated_by_last_action(true) if  b.updated_by_last_action?
      end

      def action_unload
        u = execute "unload module #{new_resource.name}" do
          command "modprobe -r #{new_resource.name}"
        end
        new_resource.updated_by_last_action(true) if  u.updated_by_last_action?
      end

      def action_blacklist
        case node['platform']
        when 'exherbo', 'ubuntu', 'arch'
          b = file path_boot do
            action :delete
          end
          new_resource.updated_by_last_action(true) if  b.updated_by_last_action?
        when 'debian', 'ubuntu'
          b = execute "blacklist module #{new_resource.name}" do
            command "sed -i '/#{new_resource.name}/d' #{path_boot}"
          end
          new_resource.updated_by_last_action(true) if  b.updated_by_last_action?
        end
        s = file path_opts do
          content "install #{new_resource.name} /bin/true\n"
          owner 'root'
          group 'root'
          mode '0644'
        end
        new_resource.updated_by_last_action(true) if  s.updated_by_last_action?
        action_unload
      end
    end
  end
end
